extends KinematicBody2D

export (int) var move = 20
export (int) var max_speed = 100
export (int) var friction = .15
export (int) var hold_offset = 0
export (int) var grab_offset = 50
export (int) var drop_offset = 15
export (int) var throw = 3
export (int) var joydar_range = 50

enum DIRECTION {LEFT, RIGHT, UP, DOWN}
var facing = UP

var velocity = Vector2(0, 0)
var held_item = null
var animation_to_play = 'idle'
var idle_animation_to_play = 'idle'
var flip = false
var joy = 0.0

func _ready():
	# Called when the node is added to the scene for the first time.
	# Initialization here
	pass

func _physics_process(delta):
	if not get_parent().is_in_game:
		$Sprite/anim.stop()
		return

	var new_animation_to_play = idle_animation_to_play

	if held_item:
		var target_hold_position = global_position + Vector2(0, hold_offset).rotated(get_facing_rotation())
		held_item.set_applied_force(Vector2(0, 0))
		held_item.add_force(
			Vector2(0, 0),
			200 * (target_hold_position - held_item.global_position)
		)
		held_item.add_force(
			Vector2(0, 0),
			-10 * held_item.linear_velocity
		)

	var acceleration = Vector2(0, 0)
	if Input.is_action_pressed('ui_right'):
		facing = RIGHT
		acceleration.x += move
		new_animation_to_play = 'walk-right'
		idle_animation_to_play = 'idle-right'
		flip = false
	elif Input.is_action_pressed('ui_left'):
		facing = LEFT
		acceleration.x -= move
		new_animation_to_play = 'walk-right'
		idle_animation_to_play = 'idle-right'
		flip = true
	else:
		acceleration.x -= friction * velocity.x
	if Input.is_action_pressed('ui_down'):
		facing = DOWN
		acceleration.y += move
		new_animation_to_play = 'walk'
		idle_animation_to_play = 'idle'
	elif Input.is_action_pressed('ui_up'):
		facing = UP
		acceleration.y -= move
		new_animation_to_play = 'walk-up'
		idle_animation_to_play = 'idle-up'
	else:
		acceleration.y -= friction * velocity.y

	velocity += acceleration
	velocity = velocity.clamped(max_speed)

	var velocity_delta = velocity * delta
	var collision = move_and_collide(velocity_delta)

	if not held_item and Input.is_action_just_pressed('ui_select'):
		var result = get_world_2d().direct_space_state.intersect_ray(
			global_position,
			global_position + Vector2(0, grab_offset).rotated(get_facing_rotation()),
			[self]
		)
		if result and result.collider.is_in_group("items"):
			pick_up(result.collider)
	elif held_item and Input.is_action_just_released('ui_select'):
		drop()

	hold_in_front()

	# if we have to flip the right idle or walk animations, flip them
	if (new_animation_to_play =='idle-right' || new_animation_to_play == 'walk-right'):
		$Sprite.flip_h = flip

	if animation_to_play != new_animation_to_play:
		animation_to_play = new_animation_to_play
		$Sprite/anim.play(animation_to_play)

	scan_for_joy()

func pick_up(item):
	held_item = item
	hold_in_front()

func drop():
	held_item.set_applied_force(Vector2(0, 0))
	var target_drop_position = Vector2(0, drop_offset).rotated(get_facing_rotation())
	var coefficient = global_position.distance_to(held_item.global_position)
	held_item.apply_impulse(
		Vector2(0, 0),
		(5 + throw * coefficient) * target_drop_position
	)

	held_item.z_index = 0
	held_item = null

func hold_in_front():
	if held_item:
		if facing == DOWN or facing == RIGHT or facing == LEFT:
			held_item.z_index = 1
		else:
			held_item.z_index = 0

func scan_for_joy():
	var space_state = get_world_2d().direct_space_state
	var joydar = CircleShape2D.new()
	joydar.set_radius(joydar_range)
	var query = Physics2DShapeQueryParameters.new()
	query.set_transform(Transform2D(0, global_position))
	query.set_exclude([self])
	query.set_shape(joydar)

	joy = 0
	var num_items = 0
	var results = space_state.intersect_shape(query)
	for result in results:
		if result.collider.is_in_group("items"):
			var item_joy = result.collider.get("joy")
			if item_joy:
				var scale = 1 - global_position.distance_to(result.collider.global_position) / joydar_range
				if scale > .9:
					scale = 1
				joy += item_joy * scale
				num_items += 1
	if num_items > 0:
		joy /= (num_items)

func get_facing_rotation():
	if facing == UP:
		return PI
	elif facing == DOWN:
		return 0
	elif facing == RIGHT:
		return -PI / 2
	elif facing == LEFT:
		return PI / 2
	return 0